@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Application Level
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus/argus/argus/ArgusApp.c#23 $
- * $DateTime: 2012/01/03 19:15:19 $
- * $Change: 2274 $
+ * $Id: //depot/argus/argus/argus/ArgusApp.c#28 $
+ * $DateTime: 2015/04/06 10:38:44 $
+ * $Change: 2973 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,14 @@
 #include <ArgusNetflow.h>
 #include <argus_def.h>
 
+#define ARGUS_WELLKNOWN_PORT    1024
+#define ARGUS_OFC_PORT          6633
+#define ARGUS_AFS_PORT_MIN      7000
+#define ARGUS_AFS_PORT_MAX      7010
+#define ARGUS_UDT_PORT          9000
+
+int ArgusControlPlaneProtocol (struct ArgusModelerStruct *, struct ArgusFlowStruct *);
+
 u_char ArgusUpdateHTTPState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
 u_char ArgusUpdateRTPState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
 void ArgusUpdateUDTState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char *);
@@ -58,6 +66,37 @@ void   ArgusParseRRRecs (struct rtcphdr *);
 void  ArgusParseRTCPBYE (struct rtcphdr *);
 void ArgusParseRTCPSDES (struct rtcphdr *);
 
+int
+ArgusControlPlaneProtocol (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flowstr)
+{
+   int retn = 0;
+
+   if (getArgusControlMonitor(model)) {
+      struct ArgusSystemFlow *flow = (struct ArgusSystemFlow *)flowstr->dsrs[ARGUS_FLOW_INDEX];
+      unsigned short proto, sport, dport;
+      unsigned char ip_p;
+
+      switch (proto = (model->ArgusThisNetworkFlowType & 0xFFFF)) {
+         case ETHERTYPE_IPV6:
+         case ETHERTYPE_IP: {
+            if (proto == ETHERTYPE_IPV6) {
+                ip_p = flow->ipv6_flow.ip_p;
+               sport = flow->ipv6_flow.sport;
+               dport = flow->ipv6_flow.dport;
+            } else {
+                ip_p = flow->ip_flow.ip_p;
+               sport = flow->ip_flow.sport;
+               dport = flow->ip_flow.dport;
+            }
+
+            if ((ip_p == IPPROTO_TCP) && ((sport == ARGUS_OFC_PORT) || (dport == ARGUS_OFC_PORT)))
+               retn = 1;
+         }
+      }
+   }
+   return (retn);
+}
+
 void
 ArgusParseRTCPSR (struct rtcphdr *rtcp)
 {
@@ -184,12 +223,6 @@ ArgusUpdateAppState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
 
                             default: {
                                if (state == ARGUS_START) {
-
-#define ARGUS_WELLKNOWN_PORT	1024
-#define ARGUS_AFS_PORT_MIN	7000
-#define ARGUS_AFS_PORT_MAX	7010
-#define ARGUS_UDT_PORT          9000
-
                                  struct rtphdr *rtp = (struct rtphdr *) model->ArgusThisUpHdr;
 
                                  if (STRUCTCAPTURED(model, *rtp)) {
