@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Modeler includes
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus/argus/argus/ArgusModeler.h#53 $
- * $DateTime: 2012/01/03 19:15:19 $
- * $Change: 2274 $
+ * $Id: //depot/argus/argus/argus/ArgusModeler.h#59 $
+ * $DateTime: 2015/04/06 10:38:44 $
+ * $Change: 2973 $
  */
 
 
@@ -81,7 +81,7 @@
 #define ARGUS_DEBUG		0xFF
 
 #define ARGUS_RECORD_WRITTEN	0x00000001
-#define ARGUSTIMEOUTQS		301
+#define ARGUSTIMEOUTQS		65534
 
 #define ARGUS_CLNS    129
 #define ARGUS_ESIS    130
@@ -235,7 +235,7 @@ struct ArgusModelerStruct {
    unsigned char ArgusFlowType, ArgusFlowKey;
    unsigned short ArgusOptionIndicator;
 
-   int ArgusInProtocol, ArgusThisDir;
+   int ArgusInProtocol, ArgusThisDir, ArgusTrackDuplicates;
 
    struct ArgusKeyStrokeConf ArgusKeyStroke;
    struct ArgusUniStats *ArgusThisStats;
@@ -258,6 +258,14 @@ struct ArgusModelerStruct {
    int ArgusThisLength;
    int ArgusThisBytes;
  
+   struct timeval ArgusGlobalTime;
+   struct timeval ArgusStartTime;
+   struct timeval ArgusNowTime;
+   struct timeval ArgusUpdateInterval;
+   struct timeval ArgusUpdateTimer;
+
+   long long ival;
+ 
    long long ArgusTotalPacket;
    long long ArgusTotalFrags;
    long long ArgusTotalIPPkts;
@@ -286,12 +294,6 @@ struct ArgusModelerStruct {
  
    long long ArgusTotalUpdates;
    long long ArgusLastUpdates;
- 
-   struct timeval ArgusGlobalTime;
-   struct timeval ArgusStartTime;
-   struct timeval ArgusNowTime;
-   struct timeval ArgusUpdateInterval;
-   struct timeval ArgusUpdateTimer;
 
    struct timeval ArgusLastPacketTimer;
    struct timeval ArgusAdjustedTimer;
@@ -448,6 +450,9 @@ void setArgusKeystrokeVariable(struct ArgusModelerStruct *, char *);
 int getArgusTunnelDiscovery (struct ArgusModelerStruct *);
 void setArgusTunnelDiscovery (struct ArgusModelerStruct *, int);
 
+int getArgusTrackDuplicates (struct ArgusModelerStruct *);
+void setArgusTrackDuplicates (struct ArgusModelerStruct *, int);
+
 void setArgusFlowKey(struct ArgusModelerStruct *, int);
 void setArgusFlowType(struct ArgusModelerStruct *, int);
 
@@ -629,6 +634,9 @@ extern void setArgusKeystrokeVariable(struct ArgusModelerStruct *, char *);
 extern int getArgusTunnelDiscovery(struct ArgusModelerStruct *);
 extern void setArgusTunnelDiscovery(struct ArgusModelerStruct *, int);
 
+extern int getArgusTrackDuplicates (struct ArgusModelerStruct *);
+extern void setArgusTrackDuplicates (struct ArgusModelerStruct *, int);
+
 extern void setArgusFlowKey(struct ArgusModelerStruct *, int);
 extern void setArgusFlowType(struct ArgusModelerStruct *, int);
 
