@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Modeler
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,9 @@
  */
 
 /*
- * $Id: //depot/argus/argus/argus/ArgusOutput.c#68 $
- * $DateTime: 2012/04/13 15:41:51 $
- * $Change: 2363 $
+ * $Id: //depot/argus/argus/argus/ArgusOutput.c#81 $
+ * $DateTime: 2015/04/06 10:38:44 $
+ * $Change: 2973 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -63,8 +63,11 @@ extern int ArgusFirstTile;
 struct timeval *getArgusMarReportInterval(struct ArgusOutputStruct *);
 void setArgusMarReportInterval(struct ArgusOutputStruct *, char *);
 
+struct ArgusRecord *ArgusGenerateInitialMar (struct ArgusOutputStruct *);
+struct ArgusRecordStruct *ArgusGenerateSupplementalMarRecord (struct ArgusOutputStruct *, unsigned char);
 struct ArgusRecordStruct *ArgusGenerateStatusMarRecord (struct ArgusOutputStruct *, unsigned char);
 
+int RaDiffTime (struct timeval *, struct timeval *, struct timeval *);
 
 struct ArgusOutputStruct *
 ArgusNewOutput (struct ArgusSourceStruct *src, struct ArgusModelerStruct *model)
@@ -108,7 +111,6 @@ ArgusNewOutput (struct ArgusSourceStruct *src, struct ArgusModelerStruct *model)
 }
 
 
-struct ArgusRecord *ArgusGenerateInitialMar (struct ArgusOutputStruct *);
 #ifdef ARGUS_SASL
 int iptostring(const struct sockaddr *, socklen_t, char *, unsigned);
 
@@ -133,11 +135,14 @@ ArgusSaslLog (void *context __attribute__((unused)), int priority, const char *m
   return SASL_OK;
 }
 
-#ifdef _LP64
-#define PLUGINDIR "/usr/lib64/sasl2"
-#else
+//#ifdef _LP64
+//#define PLUGINDIR "/usr/lib64/sasl2"
+//#else
+
 #define PLUGINDIR "/usr/lib/sasl2"
-#endif
+
+//#endif
+
 
 char *searchpath = NULL;
 
@@ -152,15 +157,25 @@ ArgusSaslGetPath(void *context __attribute__((unused)), char ** path)
     *path = PLUGINDIR;
 
 #ifdef ARGUSDEBUG
-  ArgusDebug(2, "SASL path %s", path);
+  ArgusDebug(2, "SASL path %s", *path);
 #endif
 
   return SASL_OK;
 }
 
+//typedef struct sasl_callback {
+//    /* Identifies the type of the callback function.
+//     * Mechanisms must ignore callbacks with id's they don't recognize.
+//     */
+//    unsigned long id;
+//    int (*proc)(void);   /* Callback function.  Types of arguments vary by 'id' */
+//    void *context;
+//} sasl_callback_t;
+
+typedef int (*funcptr)();
+
 static const struct sasl_callback argus_cb[] = {
-    { SASL_CB_LOG, &ArgusSaslLog, NULL },
-    { SASL_CB_GETPATH, &ArgusSaslGetPath, NULL },
+    { SASL_CB_LOG, (funcptr)&ArgusSaslLog, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 #endif
@@ -188,7 +203,7 @@ ArgusInitOutput (struct ArgusOutputStruct *output)
 
    len = ntohs(output->ArgusInitMar->hdr.len) * 4;
 
-   for (i = 0; i < ARGUS_MAX_LISTEN; i++)
+   for (i = 0; i < ARGUS_MAXLISTEN; i++)
       output->ArgusLfd[i] = -1;
 
    if (output->ArgusPortNum != 0) {
@@ -735,9 +750,19 @@ ArgusOutputProcess(void *arg)
                            }
 
                         } else {
+                           struct timeval tvbuf, *tvp = &tvbuf;
 #ifdef ARGUSDEBUG
-                           ArgusDebug (5, "ArgusOutputProcess() %d client(s) not ready fd %d sock %p start %d", output->ArgusClients->count, client->fd, client->sock, client->ArgusClientStart);
+                           ArgusDebug (5, "ArgusOutputProcess() %d client(s) not ready fd %d sock 0x%x start %d", output->ArgusClients->count, client->fd, client->sock, client->ArgusClientStart);
 #endif
+                           RaDiffTime (&output->ArgusGlobalTime, &client->startime, tvp);
+                           if (tvp->tv_sec >= ARGUS_CLIENT_STARTUP_TIMEOUT) {
+                              if (client->sock != NULL) {
+                                 ArgusDeleteSocket(output, client);
+                                 ArgusLog (LOG_WARNING, "ArgusCheckClientMessage: client %s never started: timed out", 
+                                    (client->hostname != NULL) ? client->hostname : "noname");
+                              }
+                              client->ArgusClientStart = 1;
+                           }
                         }
                         client = (void *) client->qhdr.nxt;
                      }
@@ -1066,6 +1091,7 @@ ArgusCheckClientStatus (struct ArgusOutputStruct *output, int s)
                   ArgusLog (LOG_ERR, "ArgusCheckClientStatus: ArgusCalloc %s", strerror(errno));
 
                client->fd = fd;
+               client->startime = output->ArgusGlobalTime;
 #ifdef ARGUSDEBUG
                ArgusDebug (2, "ArgusCheckClientStatus() new client\n");
 #endif
@@ -1263,13 +1289,13 @@ ArgusCheckClientMessage (struct ArgusOutputStruct *output, struct ArgusClientDat
    if ((cnt = recv (fd, buf, value, 0)) <= 0) {
       if (cnt < 0) {
 #ifdef ARGUSDEBUG
-         ArgusDebug (6, "ArgusCheckClientMessage (%p, %p) recv(%d) returned error %s\n", output, client, fd, strerror(errno));
+         ArgusDebug (5, "ArgusCheckClientMessage (%p, %p) recv(%d) returned error %s\n", output, client, fd, strerror(errno));
 #endif
          return (-1);
 
       } else {
 #ifdef ARGUSDEBUG
-         ArgusDebug (6, "ArgusCheckClientMessage (%p, %p) recv(%d) returned %d bytes\n", output, client, fd, cnt);
+         ArgusDebug (5, "ArgusCheckClientMessage (%p, %p) recv(%d) returned %d bytes\n", output, client, fd, cnt);
 #endif
          return(-3);
       }
@@ -1324,28 +1350,44 @@ ArgusCheckClientMessage (struct ArgusOutputStruct *output, struct ArgusClientDat
             case RADIUM_START: client->ArgusClientStart++; retn = 0; break;
             case RADIUM_DONE:  retn = -4; break; 
             case RADIUM_FILTER: {
-               if (ArgusFilterCompile (&client->ArgusNFFcode, &ptr[7], 1) < 0) {
+               char *reply = NULL;
+               reply = ArgusFilterCompile (&client->ArgusNFFcode, &ptr[7], 1);
+               if (reply == NULL) {
                   retn = -2;
+                  if ((cnt = send (fd, "ER", 2, 0)) != 2) {
 #ifdef ARGUSDEBUG
-                  ArgusDebug (6, "ArgusCheckClientMessage: ArgusFilter syntax error: %s\n", &ptr[7]);
+                     ArgusDebug (4, "ArgusCheckClientMessage: send error %s\n", strerror(errno));
 #endif
+                  } 
+#ifdef ARGUSDEBUG
+                  ArgusDebug (3, "ArgusCheckClientMessage: ArgusFilter filter error: %s\n", &ptr[7]);
+#endif
+               } else 
+               if (strcmp(reply, "OK")) {
+                  retn = -3;
+                  if ((cnt = send (fd, reply, 2, 0)) != 2)  {
+#ifdef ARGUSDEBUG
+                     ArgusDebug (4, "ArgusCheckClientMessage: send error %s\n", strerror(errno));
+#endif
+                  }
                } else {
+
 #ifdef ARGUSDEBUG
                   char buf[MAXSTRLEN];
                   bzero(buf, MAXSTRLEN);
                   nff_dump(&client->ArgusNFFcode, buf, MAXSTRLEN, 1);
-                  ArgusDebug (5, "ArgusInitOutput: ArgusFilterCompile returned: \n%s\n", buf);
+                  ArgusDebug (3, "ArgusInitOutput: ArgusFilterCompile returned: \n%s\n", buf);
 #endif
                   client->ArgusFilterInitialized++;
                   if ((cnt = send (fd, "OK", 2, 0)) != 2) {
                      retn = -3;
 #ifdef ARGUSDEBUG
-                     ArgusDebug (6, "ArgusCheckClientMessage: send error %s\n", strerror(errno));
+                     ArgusDebug (4, "ArgusCheckClientMessage: send error %s\n", strerror(errno));
 #endif
                   } else {
                      retn = 0;
 #ifdef ARGUSDEBUG
-                     ArgusDebug (6, "ArgusCheckClientMessage: ArgusFilter %s initialized.\n", &ptr[7]);
+                     ArgusDebug (4, "ArgusCheckClientMessage: ArgusFilter %s initialized.\n", &ptr[7]);
 #endif
                   }
                }
@@ -1387,9 +1429,10 @@ ArgusCheckClientMessage (struct ArgusOutputStruct *output, struct ArgusClientDat
 struct ArgusRecord *
 ArgusGenerateInitialMar (struct ArgusOutputStruct *output)
 {
-   struct ArgusSourceStruct *ArgusSrc;
+   struct ArgusSourceStruct *ArgusSrc, *aSrc;
    struct ArgusRecord *retn;
    struct timeval now;
+   int x, done;
 
    if ((retn = (struct ArgusRecord *) ArgusCalloc (1, sizeof(struct ArgusRecord))) == NULL)
      ArgusLog (LOG_ERR, "ArgusGenerateInitialMar(%p) ArgusCalloc error %s\n", output, strerror(errno));
@@ -1420,11 +1463,18 @@ ArgusGenerateInitialMar (struct ArgusOutputStruct *output)
    retn->argus_mar.reportInterval = getArgusFarReportInterval(output->ArgusModel)->tv_sec;
    retn->argus_mar.argusMrInterval = getArgusMarReportInterval(output)->tv_sec;
 
-   if ((ArgusSrc = output->ArgusSrc->srcs[0]) != NULL) {
-      retn->argus_mar.localnet = ArgusSrc->ArgusInterface[0].ArgusLocalNet;
-      retn->argus_mar.netmask  = ArgusSrc->ArgusInterface[0].ArgusNetMask;
+   if ((ArgusSrc = output->ArgusSrc) != NULL) {
+      for (x = 0, done = 0; x < ARGUS_MAXINTERFACE && !done; x++) {
+         if ((aSrc = ArgusSrc->srcs[x]) != NULL) {
+            if (aSrc->ArgusInterface[0].ArgusLocalNet != 0) {
+               retn->argus_mar.localnet = aSrc->ArgusInterface[0].ArgusLocalNet;
+               retn->argus_mar.netmask  = aSrc->ArgusInterface[0].ArgusNetMask;
+               done = 1;
+            }
+         }
+      }
    }
-
+  
    retn->argus_mar.nextMrSequenceNum = output->ArgusOutputSequence;
    retn->argus_mar.record_len = -1;
 
@@ -1439,6 +1489,99 @@ ArgusGenerateInitialMar (struct ArgusOutputStruct *output)
    return (retn);
 }
 
+// The supplemental mar record is designed to extend the descriptions of the sensors, their inputs,
+// providing interfaces, names, interface types, extended argus identifiers (> 4 bytes), 
+// and stats, if appropriate,
+//
+// This will be the way we provide long argus id's, IPv6, ethernet, and long strings.
+// 
+// Minimally it should provide the argusid, used in all records, and the list of input descriptions,
+// as well as the extended argus id itself.
+
+
+struct ArgusRecordStruct *
+ArgusGenerateSupplementalMarRecord (struct ArgusOutputStruct *output, unsigned char status)
+{
+   struct ArgusRecordStruct *retn = NULL;
+   struct ArgusRecord *rec = NULL;
+
+   while ((retn = (struct ArgusRecordStruct *) ArgusMallocListRecord (sizeof(*retn))) == NULL) {
+      if (output && output->ArgusInputList) {
+         if ((retn = (struct ArgusRecordStruct *) ArgusPopFrontList(output->ArgusInputList, ARGUS_LOCK)) != NULL) {
+            ArgusFreeListRecord (retn);
+         } else
+            break;
+      } else
+         break;
+   }
+
+   if (retn) {
+      struct ArgusSourceStruct *ArgusSrc = NULL, *aSrc = NULL;
+      struct timeval now;
+
+      memset(retn, 0, sizeof(*retn));
+      
+      retn->hdr.type    = ARGUS_MAR | ARGUS_VERSION;
+      retn->hdr.cause   = ARGUS_SUPPLEMENTAL;
+      retn->hdr.len     = (sizeof(struct ArgusMarSupStruct)/4) + 1;  // have size for one interface, and add as you go
+
+      rec = (struct ArgusRecord *) &retn->canon;
+      rec->hdr = retn->hdr;
+
+      rec->argus_sup.argusid = getArgusID(ArgusSourceTask);
+
+      switch (getArgusIDType(ArgusSourceTask)) {
+         case ARGUS_TYPE_STRING: rec->argus_sup.status |= ARGUS_IDIS_STRING; break;
+         case ARGUS_TYPE_INT:    rec->argus_sup.status |= ARGUS_IDIS_INT; break;
+         case ARGUS_TYPE_IPV4:   rec->argus_sup.status |= ARGUS_IDIS_IPV4; break;
+      }
+
+      gettimeofday (&now, 0L);
+
+      rec->argus_sup.startime.tv_sec  = output->ArgusLastMarUpdateTime.tv_sec;
+      rec->argus_sup.startime.tv_usec = output->ArgusLastMarUpdateTime.tv_usec;
+
+      rec->argus_sup.now.tv_sec  = now.tv_sec;
+      rec->argus_sup.now.tv_usec = now.tv_usec;
+
+      if ((ArgusSrc = output->ArgusSrc) != NULL) {
+         int x;
+         for (x = 0; x < ARGUS_MAXINTERFACE; x++) {
+            if ((aSrc = ArgusSrc->srcs[x]) != NULL) {
+               if (aSrc->ArgusInterface[0].ArgusPd != NULL) {
+                  int i;
+                  rec->argus_mar.interfaceType = pcap_datalink(aSrc->ArgusInterface[0].ArgusPd);
+                  rec->argus_mar.interfaceStatus = getArgusInterfaceStatus(aSrc);
+
+                  rec->argus_mar.pktsRcvd  = 0;
+                  rec->argus_mar.bytesRcvd = 0;
+                  rec->argus_mar.dropped   = 0;
+
+                  for (i = 0; i < ARGUS_MAXINTERFACE; i++) {
+                     rec->argus_mar.pktsRcvd  += aSrc->ArgusInterface[i].ArgusTotalPkts - 
+                                                 aSrc->ArgusInterface[i].ArgusLastPkts;
+                     rec->argus_mar.bytesRcvd += aSrc->ArgusInterface[i].ArgusTotalBytes -
+                                                 aSrc->ArgusInterface[i].ArgusLastBytes;
+                     rec->argus_mar.dropped   += aSrc->ArgusInterface[i].ArgusStat.ps_drop - 
+                                                 aSrc->ArgusInterface[i].ArgusLastDrop;
+
+                     aSrc->ArgusInterface[i].ArgusLastPkts  = aSrc->ArgusInterface[i].ArgusTotalPkts;
+                     aSrc->ArgusInterface[i].ArgusLastDrop  = aSrc->ArgusInterface[i].ArgusStat.ps_drop;
+                     aSrc->ArgusInterface[i].ArgusLastBytes = aSrc->ArgusInterface[i].ArgusTotalBytes;
+                  }
+               }
+            }
+         }
+      }
+   }
+
+#ifdef ARGUSDEBUG
+   ArgusDebug (4, "ArgusGenerateStatusMar(%p, %d) returning 0x%x", output, status, retn);
+#endif
+
+   return (retn);
+}
+
 
 struct ArgusRecordStruct *
 ArgusGenerateStatusMarRecord (struct ArgusOutputStruct *output, unsigned char status)
