@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - TCP protocol
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus/argus/argus/ArgusTcp.c#41 $
- * $DateTime: 2012/04/04 01:48:10 $
- * $Change: 2351 $
+ * $Id: //depot/argus/argus/argus/ArgusTcp.c#50 $
+ * $DateTime: 2015/04/14 18:22:14 $
+ * $Change: 3006 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,7 +60,7 @@ int ArgusUpdateTCPSequence (struct ArgusModelerStruct *, struct ArgusFlowStruct
 void ArgusTCPKeystroke (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char *);
 int ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *, struct ArgusFlowStruct *, struct tcphdr *);
 void ArgusTCPFlowRecord (struct ArgusNetworkStruct *, unsigned char);
-long long RaDiffTime (struct ArgusTime *, struct ArgusTime *, struct timeval *);
+long long ArgusDiffTime (struct ArgusTime *, struct ArgusTime *, struct timeval *);
 
 
 
@@ -316,13 +316,17 @@ ArgusInitializeTCP (struct ArgusModelerStruct *model, struct ArgusFlowStruct *fl
    bzero ((char *)tcpExt, sizeof(*tcpExt));
 
    if (fdsr) {
-      if (model->ArgusThisDir)
+      if (!(model->ArgusThisDir)) {
          fdsr->hdr.argus_dsrvl8.qual &= ~ARGUS_DIRECTION;
-      else
-         fdsr->hdr.argus_dsrvl8.qual |=  ARGUS_DIRECTION;
+         fdsr->hdr.subtype           &= ~ARGUS_REVERSE;
+      }
    }
 
+   flow->state         = model->state & ARGUS_DIRECTION;
+   model->ArgusThisDir = 1;
+
    flow->qhdr.lasttime = model->ArgusGlobalTime;
+
    ArgusUpdateFlow (model, flow, ARGUS_START, 0);
 }
 
@@ -349,7 +353,7 @@ ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *model, struct ArgusFlowSt
       ArgusThisTCPsrc->status |= ARGUS_RESET;
 
       if (state == TCPS_SYN_SENT) {
-         if ((ArgusThisTCPdst->seq == ArgusThisTCPsrc->ack))
+         if (ArgusThisTCPdst->seq == ArgusThisTCPsrc->ack)
              state = TCPS_LISTEN;
          else
              state = TCPS_CLOSING;
@@ -380,7 +384,7 @@ ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *model, struct ArgusFlowSt
                   tcpExt->status |= ARGUS_SAW_SYN_SENT;
                   ArgusThisTCPsrc->status |= ARGUS_SAW_SYN_SENT;
 
-                  if ((ArgusThisTCPdst->seq == ArgusThisTCPsrc->ack)) {
+                  if (ArgusThisTCPdst->seq == ArgusThisTCPsrc->ack) {
                      struct timeval lasttime;
                      lasttime.tv_sec  = ArgusThisTCPdst->lasttime.tv_sec;
                      lasttime.tv_usec = ArgusThisTCPdst->lasttime.tv_usec;
@@ -420,7 +424,7 @@ ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *model, struct ArgusFlowSt
                   flowstr->ArgusTimeout = ARGUS_IPTIMEOUT;
 */
                   ArgusThisTCPsrc->status |= ARGUS_CON_ESTABLISHED;
-                  if ((ArgusThisTCPsrc->seq == ArgusThisTCPdst->ack)) {
+                  if (ArgusThisTCPsrc->seq == ArgusThisTCPdst->ack) {
                      struct timeval lasttime;
                      lasttime.tv_sec  = ArgusThisTCPdst->lasttime.tv_sec;
                      lasttime.tv_usec = ArgusThisTCPdst->lasttime.tv_usec;
@@ -592,48 +596,59 @@ ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct
          ArgusThisTCPsrc->seq = newseq;
       } else {
          if (len) {
-            if (ArgusThisTCPdst->win != 0) {           /* not first packet seen in this direction */
-               if (tcp->th_seq < ArgusThisTCPdst->ack) {
-                  if ((ArgusThisTCPdst->ack - tcp->th_seq) < (ArgusThisTCPsrc->win >> ArgusThisTCPsrc->winshift)) {
-                     ArgusThisTCPsrc->retrans++;
-                     ArgusThisTCPsrc->status |= ARGUS_PKTS_RETRANS;
-                     ArgusThisTCPsrc->winbytes -= len;
-                     model->ArgusInProtocol = 0;
-                  }
-
-               } else {
-                  if (newseq > ArgusThisTCPsrc->seq) {
-                     maxseq = newseq;
-                  } else {
-                     maxseq = ArgusThisTCPsrc->seq;
-                  }
-
-                  if (ArgusThisTCPsrc->win) {
-                     int dipid;
-                     if (*tipid && ((dipid = (ipid - *tipid)) < 0) && (dipid > -5000)) {
-                        ArgusThisTCPsrc->status |= ARGUS_OUTOFORDER;
-                     } else
-                     if (ArgusThisTCPsrc->winbytes > ((maxseq - 1) - ArgusThisTCPdst->ack)) {
+            if (model->ArgusTrackDuplicates && (tipid && ((ipid != 0) && (*tipid == ipid)))) {
+/*
+               if (model->ArgusThisDir) 
+                  tcpExt->sdups++;
+               else
+                  tcpExt->ddups++;
+*/
+            } else {
+               if (ArgusThisTCPdst->win != 0) {           /* not first packet seen in this direction */
+                  if (tcp->th_seq < ArgusThisTCPdst->ack) {
+                     if ((ArgusThisTCPdst->ack - tcp->th_seq) < (ArgusThisTCPsrc->win >> ArgusThisTCPsrc->winshift)) {
                         ArgusThisTCPsrc->retrans++;
                         ArgusThisTCPsrc->status |= ARGUS_PKTS_RETRANS;
                         ArgusThisTCPsrc->winbytes -= len;
                         model->ArgusInProtocol = 0;
+                     }
 
+                  } else {
+                     if (newseq > ArgusThisTCPsrc->seq) {
+                        maxseq = newseq;
                      } else {
-                        if (newseq < ArgusThisTCPsrc->seq) {
-                           if (tcp->th_seq == (ArgusThisTCPdst->ack + 1)) {
-                              ArgusThisTCPsrc->retrans++;
-                              ArgusThisTCPsrc->status |= ARGUS_PKTS_RETRANS;
-                              ArgusThisTCPsrc->winbytes -= len;
-                           } else {
-                              ArgusThisTCPsrc->status |= ARGUS_OUTOFORDER;
+                        maxseq = ArgusThisTCPsrc->seq;
+                     }
+
+                     if (ArgusThisTCPsrc->win) {
+                        int dipid;
+                        if (*tipid && ((dipid = (ipid - *tipid)) < 0) && (dipid > -5000)) {
+                           ArgusThisTCPsrc->status |= ARGUS_OUTOFORDER;
+                        } else
+                        if (ArgusThisTCPsrc->winbytes > ((maxseq - 1) - ArgusThisTCPdst->ack)) {
+                           ArgusThisTCPsrc->retrans++;
+                           ArgusThisTCPsrc->status |= ARGUS_PKTS_RETRANS;
+                           ArgusThisTCPsrc->winbytes -= len;
+                           model->ArgusInProtocol = 0;
+
+                        } else {
+                           if (newseq < ArgusThisTCPsrc->seq) {
+                              if (tcp->th_seq == (ArgusThisTCPdst->ack + 1)) {
+                                 ArgusThisTCPsrc->retrans++;
+                                 ArgusThisTCPsrc->status |= ARGUS_PKTS_RETRANS;
+                                 ArgusThisTCPsrc->winbytes -= len;
+                              } else {
+                                 ArgusThisTCPsrc->status |= ARGUS_OUTOFORDER;
+                              }
                            }
                         }
                      }
-                  }
 
-                  ArgusThisTCPsrc->seq = maxseq;
+                     ArgusThisTCPsrc->seq = maxseq;
+                  }
                }
+
+               *tipid = ipid;
             }
 
          } else {
@@ -670,8 +685,6 @@ ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct
       }
    }
 
-   if (*tipid < ipid)
-      *tipid = ipid;
 
 /* ArgusInProtocol = 1; */
 
@@ -697,13 +710,13 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
 
       if (++flowstr->skey.n_pkts < ArgusKeyStroke->n_min) {
 #ifdef ARGUSDEBUG
-         ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld flow_n_pkts(%d) < n_min(%d)\n", 
+         ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld flow_n_pkts(%d) < n_min(%d)\n", 
             flowstr, model->ArgusTotalPacket, flowstr->skey.n_pkts, ArgusKeyStroke->n_min);
 #endif
          return;
       } else {
 #ifdef ARGUSDEBUG
-         ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld flow_n_pkts(%d)\n", 
+         ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld flow_n_pkts(%d)\n", 
             flowstr, model->ArgusTotalPacket, flowstr->skey.n_pkts);
 #endif
       }
@@ -725,7 +738,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
             if (tcpdatalen % 4) {
                flowstr->status &= ~ARGUS_SSH_MONITOR;
 #ifdef ARGUSDEBUG
-               ArgusDebug (5, "ArgusTCPKeystroke: packet %lld flow %p ssh monitor and tcpdatalen(%d) not mod 4\n", flowstr->skey.n_pkts, flowstr, tcpdatalen);
+               ArgusDebug (7, "ArgusTCPKeystroke: packet %lld flow %p ssh monitor and tcpdatalen(%d) not mod 4\n", flowstr->skey.n_pkts, flowstr, tcpdatalen);
 #endif
                return;
             }
@@ -744,7 +757,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                if ((tcpdatalen >= ArgusKeyStroke->dc_min) && (tcpdatalen <= ArgusKeyStroke->dc_max)) {
                   int i = 0, found = 0;
 #ifdef ARGUSDEBUG
-                  ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld client packet seq(%u) size(%d) OK\n", flowstr, flowstr->skey.n_pkts, tcp->th_seq, tcpdatalen);
+                  ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld client packet seq(%u) size(%d) OK\n", flowstr, flowstr->skey.n_pkts, tcp->th_seq, tcpdatalen);
 #endif
                   for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS && !found; i++) {
                      struct ArgusKeyStrokePacket *pkt = &flowstr->skey.data.pkts[i];
@@ -753,7 +766,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                         struct timeval tvpbuf, *tvp = &tvpbuf;
                         pkt->ts.tv_sec  = model->ArgusGlobalTime.tv_sec;
                         pkt->ts.tv_usec = model->ArgusGlobalTime.tv_usec;
-                        pkt->intpkt = RaDiffTime (&pkt->ts, &flowstr->skey.prev_c_ts, tvp);
+                        pkt->intpkt = ArgusDiffTime (&pkt->ts, &flowstr->skey.prev_c_ts, tvp);
                         pkt->seq = tcp->th_seq + tcpdatalen;
                         pkt->n_pno = flowstr->skey.n_pkts;
                         found = 1;
@@ -772,7 +785,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                      }
                      if (npkt < 0) {  // nothing but tentatives, so lets clear the tentative at index = 0
 #ifdef ARGUSDEBUG
-                        ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld no non-tentative packets to reject, clearing first\n", flowstr, flowstr->skey.n_pkts);
+                        ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld no non-tentative packets to reject, clearing first\n", flowstr, flowstr->skey.n_pkts);
 #endif
                         npkt = 0;
                      }
@@ -780,11 +793,11 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                         struct ArgusKeyStrokePacket *pkt = &flowstr->skey.data.pkts[npkt];
                         struct timeval tvpbuf, *tvp = &tvpbuf;
 #ifdef ARGUSDEBUG
-                        ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld clearing client %d seq(%u)\n", flowstr, flowstr->skey.n_pkts, npkt, pkt->seq);
+                        ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld clearing client %d seq(%u)\n", flowstr, flowstr->skey.n_pkts, npkt, pkt->seq);
 #endif
                         pkt->ts.tv_sec  = model->ArgusGlobalTime.tv_sec;
                         pkt->ts.tv_usec = model->ArgusGlobalTime.tv_usec;
-                        pkt->intpkt = RaDiffTime (&pkt->ts, &flowstr->skey.prev_c_ts, tvp);
+                        pkt->intpkt = ArgusDiffTime (&pkt->ts, &flowstr->skey.prev_c_ts, tvp);
                         pkt->seq = tcp->th_seq + tcpdatalen;
                         pkt->n_pno = flowstr->skey.n_pkts;
 
@@ -793,7 +806,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                }
 #ifdef ARGUSDEBUG
                  else {
-                  ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld client packet size(%d) NOK reset\n", flowstr, flowstr->skey.n_pkts, tcpdatalen);
+                  ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld client packet size(%d) NOK reset\n", flowstr, flowstr->skey.n_pkts, tcpdatalen);
                }
 #endif
                break;
@@ -820,21 +833,21 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                         struct timeval tvpbuf, *tvp = &tvpbuf;
 
                         if (pkt->intpkt >= ArgusKeyStroke->ic_min) {
-                           long long slint = RaDiffTime (&stime, &flowstr->skey.prev_s_ts, tvp);
+                           long long slint = ArgusDiffTime (&stime, &flowstr->skey.prev_s_ts, tvp);
 
                            float ic_ratio = (pkt->intpkt * 1.0) / (slint * 1.0);
                            if ((ic_ratio >= ArgusKeyStroke->icr_min) && (ic_ratio <= ArgusKeyStroke->icr_max)) {
                               if ((pkt->n_pno - flowstr->skey.prev_pno) <= ArgusKeyStroke->gpc_max) {
                                  int i;
 #ifdef ARGUSDEBUG
-                                 ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server keystroke ", flowstr, flowstr->skey.n_pkts);
+                                 ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld server keystroke ", flowstr, flowstr->skey.n_pkts);
 #endif
                                  flowstr->skey.n_strokes++;
                                  for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS; i++) {
                                     struct ArgusKeyStrokePacket *tpkt = &flowstr->skey.data.pkts[i];
                                     if (tpkt->status == ARGUS_KEYSTROKE_TENTATIVE) {
 #ifdef ARGUSDEBUG
-                                       ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld TENTATIVE packet %lld keystroke ", flowstr, flowstr->skey.n_pkts, tpkt->n_pno);
+                                       ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld TENTATIVE packet %lld keystroke ", flowstr, flowstr->skey.n_pkts, tpkt->n_pno);
 #endif
                                        flowstr->skey.n_strokes++;
                                        bzero(tpkt, sizeof(*tpkt));
@@ -853,7 +866,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                                  }
                                  pkt->status = ARGUS_KEYSTROKE_TENTATIVE;
 #ifdef ARGUSDEBUG
-                                 ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server packet pc_gap(%d) > gpc_max(%d) TENTATIVE\n", 
+                                 ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld server packet pc_gap(%d) > gpc_max(%d) TENTATIVE\n", 
                                        flowstr, flowstr->skey.n_pkts, (pkt->n_pno - flowstr->skey.prev_pno) , ArgusKeyStroke->gpc_max);
 #endif
                                  flowstr->skey.prev_pno  = pkt->n_pno;
@@ -865,27 +878,27 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                            } 
 #ifdef ARGUSDEBUG
                            else {
-                                 ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server ic_ratio(%f) out of range\n", 
+                                 ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld server ic_ratio(%f) out of range\n", 
                                     flowstr, flowstr->skey.n_pkts, ic_ratio);
                            }
 #endif
                         } 
 #ifdef ARGUSDEBUG
                         else {
-                           ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld rejected server IPA(%lld) < ic_min(%d)\n", 
+                           ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld rejected server IPA(%lld) < ic_min(%d)\n", 
                                     flowstr, flowstr->skey.n_pkts, pkt->intpkt, ArgusKeyStroke->ic_min);
                         }
 #endif
                      } 
 #ifdef ARGUSDEBUG
                      else {
-                        ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld rejected server packet size(%d)\n", flowstr, flowstr->skey.n_pkts, tcpdatalen);
+                        ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld rejected server packet size(%d)\n", flowstr, flowstr->skey.n_pkts, tcpdatalen);
                      }
 #endif
                   } 
 #ifdef ARGUSDEBUG
                   else {
-                     ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld rejected server packet flow s_gap(%d) > gs_gap(%d)\n", 
+                     ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld rejected server packet flow s_gap(%d) > gs_gap(%d)\n", 
                            flowstr, flowstr->skey.n_pkts, (flowstr->skey.n_pkts - pkt->n_pno), ArgusKeyStroke->gs_max);
                   }
 #endif
@@ -900,7 +913,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
                } 
 #ifdef ARGUSDEBUG
                else {
-                  ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server packet ack(%u) no match\n", 
+                  ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld server packet ack(%u) no match\n", 
                      flowstr, flowstr->skey.n_pkts, tcp->th_ack);
                }
 #endif
@@ -910,7 +923,7 @@ ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flo
       } 
 #ifdef ARGUSDEBUG
       else {
-         ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld %s packet rejected tcpdatalen(%d)\n", 
+         ArgusDebug (7, "ArgusTCPKeystroke: flow %p packet %lld %s packet rejected tcpdatalen(%d)\n", 
                flowstr, flowstr->skey.n_pkts, (model->ArgusThisDir ? "server" : "client"), tcpdatalen);
       }
 #endif
