@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Utilities
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus/argus/argus/ArgusUtil.c#70 $
- * $DateTime: 2012/03/15 00:43:34 $
- * $Change: 2332 $
+ * $Id: //depot/argus/argus/argus/ArgusUtil.c#87 $
+ * $DateTime: 2015/04/06 10:38:44 $
+ * $Change: 2973 $
  */
 
 /* ArgusUtil.c */
@@ -586,11 +586,11 @@ ArgusRemoveFromQueue(struct ArgusQueueStruct *queue, struct ArgusQueueHeader *ob
    struct ArgusQueueHeader *retn = NULL;
 
    if ((queue != NULL) && (obj != NULL)) {
-      if (obj->queue == queue) {
 #if defined(ARGUS_THREADS)
-         if (type == ARGUS_LOCK)
-            pthread_mutex_lock(&queue->lock); 
+      if (type == ARGUS_LOCK)
+         pthread_mutex_lock(&queue->lock); 
 #endif
+      if (obj->queue == queue) {
          if (queue->count) {
             queue->count--;
 
@@ -608,19 +608,17 @@ ArgusRemoveFromQueue(struct ArgusQueueStruct *queue, struct ArgusQueueHeader *ob
                queue->end   = NULL;
             }
          }
-#if defined(ARGUS_THREADS)
-         if (type == ARGUS_LOCK)
-            pthread_mutex_unlock(&queue->lock); 
-#endif
          obj->prv = NULL;
          obj->nxt = NULL;
          obj->queue = NULL;
          retn = obj;
+      }
 
-      } else
-         ArgusLog (LOG_ERR, "ArgusRemoveFromQueue(%p, %p) obj not in queue\n", queue, obj);
-   } else
-      ArgusLog (LOG_ERR, "ArgusRemoveFromQueue(%p, %p) parameter error\n", queue, obj);
+#if defined(ARGUS_THREADS)
+      if (type == ARGUS_LOCK)
+         pthread_mutex_unlock(&queue->lock); 
+#endif
+   }
 
 #ifdef ARGUSDEBUG
    ArgusDebug (10, "ArgusRemoveFromQueue (%p, %p) returning %p\n", queue, obj, obj);
@@ -666,16 +664,20 @@ ArgusCheckTimeout(struct ArgusModelerStruct *model, struct timeval *ts, struct t
    long long diff = 0, tdiff = 0;
    int retn;
 
-   if ((timeout->tv_sec > 0) || (timeout->tv_usec > 0)) {
-      diff  = ArgusTimeDiff (&model->ArgusGlobalTime, ts);
-      tdiff = (timeout->tv_sec * 1000000LL + timeout->tv_usec);
+   if (timeout->tv_sec < 0)  // if timeout is set to less that zero, then we never timeout.
+      retn = 0;
+   else {
+      if ((timeout->tv_sec > 0) || (timeout->tv_usec > 0)) {
+         diff  = ArgusTimeDiff (&model->ArgusGlobalTime, ts);
+         tdiff = (timeout->tv_sec * 1000000LL + timeout->tv_usec);
 
-      if (diff >= tdiff)
+         if (diff >= tdiff)
+            retn = 1;
+         else
+            retn = 0;
+      } else
          retn = 1;
-      else
-         retn = 0;
-   } else
-      retn = 1;
+   }
 
 #ifdef ARGUSDEBUG
    ArgusDebug (11, "ArgusCheckTimeout (%p, %d.%06d, %d.%06d) diff %f returning %d\n", model, ts->tv_sec, ts->tv_usec,
@@ -752,15 +754,14 @@ ArgusDeleteObject(struct ArgusFlowStruct *obj)
 int
 ArgusUpdateTime (struct ArgusModelerStruct *model)
 {
-   int retn = 0;
+   long long ival = model->ival;
    long long diff;
-   long long ival;
+   int retn = 0;
 
    if (model->ArgusUpdateTimer.tv_sec == 0) 
       model->ArgusUpdateTimer = model->ArgusGlobalTime;
 
    diff = ArgusTimeDiff(&model->ArgusGlobalTime, &model->ArgusUpdateTimer);
-   ival = ((model->ArgusUpdateInterval.tv_sec * 1000000LL) + model->ArgusUpdateInterval.tv_usec);
 
    if (diff >= 0) {
       retn = 1;
@@ -935,16 +936,36 @@ ArgusCreateFlowKey (struct ArgusModelerStruct *model, struct ArgusSystemFlow *fl
       hstruct->len = len;
 
       if (model->ArgusFlowKey & ARGUS_FLOW_KEY_VLAN) {
-         *key++ = model->ArgusThisPacket8021QEncaps;
+         *key++ = model->ArgusThisPacket8021QEncaps & 0x0FFF;
          hstruct->len++;
       }
    
-      if (model->ArgusFlowKey & (ARGUS_FLOW_KEY_LOCAL_MPLS |
-                                 ARGUS_FLOW_KEY_COMPLETE_MPLS)) {
+      if (model->ArgusFlowKey & (ARGUS_FLOW_KEY_LOCAL_MPLS | ARGUS_FLOW_KEY_COMPLETE_MPLS)) {
          *key++ = model->ArgusThisMplsLabel;
          hstruct->len++;
       }
 
+      if (model->ArgusFlowKey & (ARGUS_FLOW_KEY_LAYER_2 | ARGUS_FLOW_KEY_LAYER_2_MATRIX)) {
+         struct ether_header *ep = model->ArgusThisEpHdr;
+         if (ep) {
+            int klen = (sizeof(*ep) + (sizeof(*key) - 1)) / sizeof(*key);
+            if (model->state & ARGUS_DIRECTION) {
+#ifndef ETH_ALEN
+#define ETH_ALEN   6
+#endif
+               char *kptr = (char *) key;
+               bcopy ((char *)&ep->ether_shost, kptr, ETH_ALEN); kptr += ETH_ALEN;
+               bcopy ((char *)&ep->ether_dhost, kptr, ETH_ALEN); kptr += ETH_ALEN;
+               bcopy ((char *)&ep->ether_type,  kptr, sizeof(ep->ether_type));
+
+            } else
+               bcopy (ep, key, sizeof(*ep));
+
+            key += klen;
+            hstruct->len += klen;
+         }
+      }
+
       ptr = hstruct->key;
 
       for (i = 0; i < hstruct->len; i++)
@@ -1152,7 +1173,7 @@ ArgusZeroRecord (struct ArgusFlowStruct *flow)
 
             case ARGUS_METRIC_INDEX: {
                struct ArgusMetricStruct *metric = (void *) flow->dsrs[i];
-               bzero(metric, sizeof(*metric));
+               bzero((&metric->hdr + 1), sizeof(*metric) - 4);
                break;
             }
 
@@ -1369,9 +1390,6 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
    struct statfs statfsbuf;
 #endif
 
-#if defined(ARGUS_THREADS)
-      pthread_mutex_lock(&list->lock);
-#endif
       if (ArgusListEmpty (list) && (asock->rec == NULL)) {
 #ifdef ARGUSDEBUG
          if (asock->writen || asock->length)
@@ -1380,14 +1398,14 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
          if (client->host == NULL) {
          if (!(output->ArgusWriteStdOut) && (asock->filename)) {
             if (asock->lastwrite.tv_sec < output->ArgusModel->ArgusGlobalTime.tv_sec) {
-               if (((stat (asock->filename, &statbuf)) < 0) || (asock->fd != -1) || (ArgusCloseFile)) {
+               if (((stat (asock->filename, &statbuf)) < 0) || (ArgusCloseFile)) {
                   if (asock->fd != -1)
                      close(asock->fd);
                   if ((asock->fd = open (asock->filename, O_WRONLY|O_APPEND|O_CREAT|O_NONBLOCK, 0x1a4)) < 0)
                      ArgusLog (LOG_ERR, "ArgusWriteSocket: open(%s, flags, 0x1a4) failed %s\n",
                         asock->filename, strerror(errno));
 #ifdef ARGUSDEBUG
-                     ArgusDebug (6, "ArgusWriteSocket: created outfile %s\n", asock->filename);
+                     ArgusDebug (2, "ArgusWriteSocket: created outfile %s\n", asock->filename);
 #endif
                }
 
@@ -1545,6 +1563,9 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
       } else {
          if (list->count >= ArgusMaxListLength) {
             if (ArgusWriteOutSocket(output, client) < 0) {
+#if defined(ARGUS_THREADS)
+               pthread_mutex_lock(&list->lock);
+#endif
                if (list->count >= ArgusMaxListLength) {
                   struct ArgusRecordStruct *trec;
                   int i;
@@ -1555,21 +1576,21 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
                      if ((trec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_NOLOCK)) != NULL)
                         ArgusFreeListRecord(trec);
                }
+#if defined(ARGUS_THREADS)
+               pthread_mutex_unlock(&list->lock);
+#endif
             }
          }
 
          if (asock->rec == NULL)
-            asock->rec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_NOLOCK);
+            asock->rec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_LOCK);
 
 #ifdef ARGUSDEBUG
          ArgusDebug (6, "ArgusWriteSocket (%p, %p, %p) schedule record\n", output, asock, rec);
 #endif
-         ArgusPushBackList (list, (struct ArgusListRecord *) ArgusCopyRecordStruct(rec), ARGUS_NOLOCK);
+         ArgusPushBackList (list, (struct ArgusListRecord *) ArgusCopyRecordStruct(rec), ARGUS_LOCK);
          retn = 0;
       }
-#if defined(ARGUS_THREADS)
-      pthread_mutex_unlock(&list->lock);
-#endif
 
 #ifdef ARGUSDEBUG
       ArgusDebug (6, "ArgusWriteSocket (%p, %p, %p) returning %d\n", output, asock, rec, retn);
@@ -1616,7 +1637,7 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
                               ArgusLog (LOG_ERR, "ArgusWriteSocket: open(%s, O_WRONLY|O_APPEND|O_CREAT|O_NONBLOCK, 0x1a4) failed %s\n",
                                          asock->filename, strerror(errno));
 #ifdef ARGUSDEBUG
-                           ArgusDebug (6, "ArgusWriteOutSocket: created outfile %s\n", asock->filename);
+                           ArgusDebug (2, "ArgusWriteOutSocket: created outfile %s\n", asock->filename);
 #endif
                         }
 
@@ -2030,10 +2051,52 @@ RaMaxTime (struct timeval *s1, struct timeval *s2)
    return (retn);
 }
 
-long long RaDiffTime (struct ArgusTime *, struct ArgusTime *, struct timeval *);
+
+float
+RaDeltaFloatTime (struct timeval *s1, struct timeval *s2)
+{
+   float retn = 0.0;
+
+   if (s1 && s2) {
+      double v1 = (s1->tv_sec * 1.0) + (s1->tv_usec / 1000000.0);
+      double v2 = (s2->tv_sec * 1.0) + (s2->tv_usec / 1000000.0);
+
+      retn = v1 - v2;
+   }
+
+   return (retn);
+}
+
+int
+RaDiffTime (struct timeval *s1, struct timeval *s2, struct timeval *diff)
+{
+   int retn = 0;
+
+   if (s1 && s2 && diff) {
+      bzero ((char *)diff, sizeof(*diff));
+
+      double v1 = (s1->tv_sec * 1.0) + (s1->tv_usec / 1000000.0);
+      double v2 = (s2->tv_sec * 1.0) + (s2->tv_usec / 1000000.0);
+      double f, i;
+
+      v1 -= v2;
+
+      f = modf(v1, &i);
+
+      diff->tv_sec  = i;
+      diff->tv_usec = f * 1000000;
+
+      retn = 1;
+   }
+
+   return (retn);
+}
+
+
+long long ArgusDiffTime (struct ArgusTime *, struct ArgusTime *, struct timeval *);
 
 long long
-RaDiffTime (struct ArgusTime *s1, struct ArgusTime *s2, struct timeval *diff)
+ArgusDiffTime (struct ArgusTime *s1, struct ArgusTime *s2, struct timeval *diff)
 {
    long long v1 = 0, v2 = 0;
 
@@ -2687,3 +2750,21 @@ ArgusFetchRate (struct ArgusRecordStruct *ns)
    retn = r1;
    return (retn);
 }
+
+double
+ArgusFetchAppByteRatio (struct ArgusRecordStruct *ns)
+{
+   struct ArgusMetricStruct *m1 = NULL;
+   double retn =  0.0;
+
+   if ((m1 = (struct ArgusMetricStruct *) ns->dsrs[ARGUS_METRIC_INDEX]) != NULL) {
+      double nvalue = (m1->src.appbytes - m1->dst.appbytes) * 1.0;
+      double dvalue = (m1->src.appbytes + m1->dst.appbytes) * 1.0;
+
+      if (dvalue > 0)
+         retn = nvalue / dvalue;
+      else
+         retn = -0.0;
+   }
+   return (retn);
+}
