@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - main argus processing
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus/argus/argus/argus.c#89 $
- * $DateTime: 2012/04/13 15:47:27 $
- * $Change: 2364 $
+ * $Id: //depot/argus/argus/argus/argus.c#100 $
+ * $DateTime: 2015/04/13 00:43:29 $
+ * $Change: 2982 $
  */
 
 /*
@@ -102,6 +102,8 @@ usage(void)
 #endif
 
    fprintf (stdout, "         -e <value>              specify Argus Identifier <value>.\n");
+   fprintf (stdout, "         -f                      don't stop when EOF is seen, but wait for more\n");
+   fprintf (stdout, "                                 packets to arrive in packet capture file.\n");
    fprintf (stdout, "         -F <conffile>           read configuration from <conffile>.\n");
    fprintf (stdout, "         -h                      print help.\n");
    fprintf (stdout, "         -i <interface>          specify interface to use as a packet source.\n");
@@ -173,6 +175,7 @@ char *ArgusResourceEnvStr [] = {
 static char ArgusPidFileName[MAXPATHNAMELEN];
 char * ArgusCreatePIDFile (struct ArgusSourceStruct *, char *, char *);
 void setArgusEventDataRecord (char *);
+extern void setArgusPcapBufSize (struct ArgusSourceStruct *, int);
 
 #define ARGUS_MAX_INSTANCES	5
 
@@ -182,7 +185,7 @@ ArgusCreatePIDFile (struct ArgusSourceStruct *src, char *pidpath, char *appname)
 {
    FILE *fd;
    char pidstrbuf[128], *pidstr = pidstrbuf;
-   char *retn = NULL, *dev, *devstr;
+   char *retn = NULL, *dev = NULL, *devstr = NULL;
    int i, pid;
    struct stat statbuf;
 
@@ -334,6 +337,7 @@ main (int argc, char *argv[])
                            break;
                      }
                      break;
+
                   case 'X': 
                   case 'F': 
                      doconf++; 
@@ -377,7 +381,7 @@ main (int argc, char *argv[])
 
    optind = 1, opterr = 0;
 
-   while ((op = getopt (argc, argv, "AbB:c:CdD:e:F:g:i:JmM:N:OP:pRr:S:s:tT:u:U:w:XZh")) != EOF) {
+   while ((op = getopt (argc, argv, "AbB:c:CdD:e:fF:g:i:JmM:N:OP:pRr:S:s:tT:u:U:w:XZh")) != EOF) {
       switch (op) {
          case 'A': setArgusAflag(ArgusModel, 1); break;
          case 'b': setArgusbpflag (ArgusSourceTask, 1); break;
@@ -395,6 +399,17 @@ main (int argc, char *argv[])
          case 'd': ArgusDaemon = ArgusDaemon ? 0 : 1; break;
          case 'D': setArgusdflag (ArgusModel, atoi (optarg)); break;
          case 'e': ArgusParseSourceID(ArgusSourceTask, optarg); break;
+         case 'f': setArgusfflag (ArgusSourceTask, 1); break;
+         case 'F': ArgusParseResourceFile (ArgusModel, optarg); break;
+
+         case 'g': {
+            struct group *gr;
+            if ((gr = getgrnam(optarg)) == NULL)
+                ArgusLog (LOG_ERR, "unknown group \"%s\"\n", optarg);
+            new_gid = gr->gr_gid;
+            endgrent();
+            break;
+         }
 
          case 'i': {
             char Istr[1024], *Iptr = Istr;
@@ -430,16 +445,6 @@ main (int argc, char *argv[])
             break;
          }
 
-         case 'g': {
-            struct group *gr;
-            if ((gr = getgrnam(optarg)) == NULL)
-                ArgusLog (LOG_ERR, "unknown group \"%s\"\n", optarg);
-            new_gid = gr->gr_gid;
-            endgrent();
-            break;
-         }
-
-         case 'F': ArgusParseResourceFile (ArgusModel, optarg); break;
          case 'J': setArgusGenerateTime  (ArgusModel, 1); break;
          case 'm': setArgusmflag (ArgusModel, 1); break;
          case 'M': setArgusMarReportInterval (ArgusOutputTask, optarg); break;
@@ -541,6 +546,7 @@ main (int argc, char *argv[])
             }
          case 'X': clearArgusConfiguration (ArgusModel); break;
          case 'Z': setArgusGeneratePacketSize(ArgusModel, 1); break;
+
          case 'h':
          default:
             usage ();
@@ -556,9 +562,11 @@ main (int argc, char *argv[])
    if (getArgusrfile(ArgusSourceTask) != NULL) {
       if (!(getArgusRealTime(ArgusSourceTask))) {
          ArgusDaemon = 0;
-         setArgusPortNum(ArgusOutputTask, 0);
          setArgusBindAddr(ArgusOutputTask, NULL);
          ArgusSourceTask->ArgusReadingOffLine++;
+         if (getArgusfflag (ArgusSourceTask) == 0) {
+            setArgusPortNum(ArgusOutputTask, 0);
+         }
       }
    }
 
@@ -628,7 +636,9 @@ main (int argc, char *argv[])
       ArgusLog (LOG_ERR, "pthreads set sched param error");
 #endif
 #else
+#if defined(__sun__)
    pthread_attr_setschedpolicy(ArgusAttr, SCHED_RR);
+#endif
 #endif
 
    pthread_attr_setdetachstate(ArgusAttr, PTHREAD_CREATE_JOINABLE);
@@ -757,7 +767,8 @@ ArgusComplete ()
 
    ArgusFree(ArgusOutputTask);
    ArgusFree(ArgusModel);
-   ArgusFree(ArgusSourceTask);
+
+   ArgusDeleteSource(ArgusSourceTask);
 
 #if defined(ARGUSPERFMETRICS)
    len = strlen(ArgusProgramName);
@@ -950,7 +961,7 @@ getArguspidflag ()
    return (pidflag);
 }
 
-#define ARGUS_RCITEMS				48
+#define ARGUS_RCITEMS				51
 
 #define ARGUS_DAEMON				0
 #define ARGUS_MONITOR_ID			1
@@ -1001,9 +1012,11 @@ getArguspidflag ()
 #define ARGUS_FRAG_TIMEOUT			46
 #define ARGUS_ARP_TIMEOUT			47
 #define ARGUS_OTHER_TIMEOUT			48
+#define ARGUS_TRACK_DUPLICATES			49
+#define ARGUS_PCAP_BUF_SIZE			50
 
 
-char *ArgusResourceFileStr [] = {
+char *ArgusResourceFileStr [ARGUS_RCITEMS] = {
    "ARGUS_DAEMON=",
    "ARGUS_MONITOR_ID=",
    "ARGUS_ACCESS_PORT=",
@@ -1053,6 +1066,8 @@ char *ArgusResourceFileStr [] = {
    "ARGUS_FRAG_TIMEOUT=",
    "ARGUS_ARP_TIMEOUT=",
    "ARGUS_OTHER_TIMEOUT=",
+   "ARGUS_TRACK_DUPLICATES=",
+   "ARGUS_PCAP_BUF_SIZE=",
 };
 
 
@@ -1369,11 +1384,8 @@ ArgusParseResourceFile (struct ArgusModelerStruct *model, char *file)
                               if (!(strncasecmp(tok, "CLASSIC_5_TUPLE", 14)))
                                  setArgusFlowKey (model, ARGUS_FLOW_KEY_CLASSIC5TUPLE);
                               else
-                              if (!(strncasecmp(tok, "LAYER_2_MATRIX", 14)))
-                                 setArgusFlowKey (model, ARGUS_FLOW_KEY_LAYER_2_MATRIX);
-                              else
-                              if (!(strncasecmp(tok, "LAYER_3_MATRIX", 14)))
-                                 setArgusFlowKey (model, ARGUS_FLOW_KEY_LAYER_3_MATRIX);
+                              if (!(strncasecmp(tok, "LAYER_2", 10)))
+                                 setArgusFlowKey (model, ARGUS_FLOW_KEY_LAYER_2);
                               else
                               if (!(strncasecmp(tok, "LOCAL_MPLS", 10)))
                                  setArgusFlowKey (model, ARGUS_FLOW_KEY_LOCAL_MPLS);
@@ -1383,6 +1395,12 @@ ArgusParseResourceFile (struct ArgusModelerStruct *model, char *file)
                               else
                               if (!(strncasecmp(tok, "VLAN", 4)))
                                  setArgusFlowKey (model, ARGUS_FLOW_KEY_VLAN);
+                              else
+                              if (!(strncasecmp(tok, "LAYER_2_MATRIX", 14)))
+                                 setArgusFlowKey (model, ARGUS_FLOW_KEY_LAYER_2_MATRIX);
+                              else
+                              if (!(strncasecmp(tok, "LAYER_3_MATRIX", 14)))
+                                 setArgusFlowKey (model, ARGUS_FLOW_KEY_LAYER_3_MATRIX);
 
                               optarg = NULL;
                            }
@@ -1500,6 +1518,7 @@ ArgusParseResourceFile (struct ArgusModelerStruct *model, char *file)
                            }
                            break;
                         }
+
                         case ARGUS_TUNNEL_DISCOVERY: {
                            if (!(strncasecmp(optarg, "yes", 3)))
                               setArgusTunnelDiscovery(model, 1);
@@ -1507,7 +1526,13 @@ ArgusParseResourceFile (struct ArgusModelerStruct *model, char *file)
                               setArgusTunnelDiscovery(model, 0);
                            break;
                         }
-
+                        case ARGUS_TRACK_DUPLICATES: {
+                           if (!(strncasecmp(optarg, "yes", 3)))
+                              setArgusTrackDuplicates(model, 1);
+                           else
+                              setArgusTrackDuplicates(model, 0);
+                           break;
+                        }
                         case ARGUS_IP_TIMEOUT: {
                            setArgusIpTimeout (model, atoi(optarg));
                            break;
@@ -1536,6 +1561,18 @@ ArgusParseResourceFile (struct ArgusModelerStruct *model, char *file)
                            setArgusOtherTimeout (model, atoi(optarg));
                            break;
                         }
+
+                        case ARGUS_PCAP_BUF_SIZE: {
+                           int size = atoi(optarg);
+                           if (size > 0) {
+                              if (strchr(optarg, 'K')) size *= 1000;
+                              if (strchr(optarg, 'M')) size *= 1000000;
+                              if (strchr(optarg, 'G')) size *= 1000000000;
+
+                              setArgusPcapBufSize (ArgusSourceTask, size);
+                           }
+                           break;
+                        }
                      }
 
                      done = 1;
@@ -1584,13 +1621,11 @@ clearArgusConfiguration (struct ArgusModelerStruct *model)
    setArgusTimeReport(model, 0);
 
    if (ArgusSourceTask->ArgusWriteOutPacketFile) {
-      if (ArgusSourceTask->ArgusWriteOutPacketFile) {
-         if (ArgusSourceTask->ArgusPcapOutFile != NULL) {
-            pcap_dump_close(ArgusSourceTask->ArgusPcapOutFile);
-            ArgusSourceTask->ArgusPcapOutFile = NULL;
-         }
-         ArgusSourceTask->ArgusWriteOutPacketFile = NULL;
+      if (ArgusSourceTask->ArgusPcapOutFile != NULL) {
+         pcap_dump_close(ArgusSourceTask->ArgusPcapOutFile);
+         ArgusSourceTask->ArgusPcapOutFile = NULL;
       }
+      ArgusSourceTask->ArgusWriteOutPacketFile = NULL;
    }
 
    if (ArgusSourceTask->ArgusInputFilter) {
