@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus/argus/include/argus_def.h#39 $
- * $DateTime: 2012/01/03 19:15:19 $
- * $Change: 2274 $
+ * $Id: //depot/argus/argus/include/argus_def.h#47 $
+ * $DateTime: 2015/04/06 10:38:44 $
+ * $Change: 2973 $
  */
 
 /* Argus_def.h */
@@ -117,7 +117,7 @@ extern "C" {
     0                   1                   2                   3   
     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-   |  Type | Cause |  Vers |  Opt  |             Length            |
+   |  Type |  Vers | Cause |  Opt  |             Length            |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 
 */
@@ -200,6 +200,7 @@ extern "C" {
 #define ARGUS_CLOSED				0x70   /* Argus Initiates Shutdown */
 
 #define ARGUS_ERROR				0x80   /* Error - Major Problem */
+#define ARGUS_SUPPLEMENTAL			0x90   /* Argus Supplemental Record */
 
 /*
    Argus  Record Header Option Field
@@ -222,6 +223,9 @@ extern "C" {
 /*
    Argus Status Option Messages
 */
+/*
+   Argus Supplemental Option Messages
+*/
 /*
    Argus Stop Option Messages
 */
@@ -608,6 +612,8 @@ extern "C" {
 #define ARGUS_TIME_DST_START                    0x20
 #define ARGUS_TIME_DST_END                      0x40
 
+#define ARGUS_TIME_MASK (ARGUS_TIME_SRC_START | ARGUS_TIME_DST_START | ARGUS_TIME_SRC_END | ARGUS_TIME_DST_END)
+
 
 /* Argus Flow Descriptor DSR Qualifiers */
  
@@ -768,14 +774,16 @@ extern "C" {
 #define ARGUS_FLOW_MPLS                         0x04
 #define ARGUS_FLOW_VLAN                         0x05
 #define ARGUS_FLOW_ARP                          0x06
+#define ARGUS_FLOW_LAYER_2                      0x07
 
-#define ARGUS_FLOW_KEYS                         0x00000006
+#define ARGUS_FLOW_KEYS                         0x00000007
 #define ARGUS_FLOW_KEY_CLASSIC5TUPLE            0x00000001
 #define ARGUS_FLOW_KEY_LAYER_2_MATRIX           0x00000002
 #define ARGUS_FLOW_KEY_LAYER_3_MATRIX           0x00000004
 #define ARGUS_FLOW_KEY_LOCAL_MPLS               0x00000008
 #define ARGUS_FLOW_KEY_COMPLETE_MPLS            0x00000010
 #define ARGUS_FLOW_KEY_VLAN                     0x00000020
+#define ARGUS_FLOW_KEY_LAYER_2                  0x00000040
 
 
 #define ARGUS_FLOW_COMPOSITE                    0x08
@@ -783,7 +791,7 @@ extern "C" {
 #define ARGUS_FLOW_ETH_IANA			0x11  /* this is the layer 3 protocol ids */
 #define ARGUS_FLOW_IP_IANA			0x12  /* this is the layer 4 protocol/nexthop number */
 
-#define ARGUS_FLOW_RMON				0x40  /* this is the layer 4 protocol/nexthop number */
+#define ARGUS_FLOW_RMON				0x40  /* this has been created usig RMON options */
 
 /* Argus Flow Descriptor DSR Qualifiers */
 #define ARGUS_SYSTEM_FLOW			0x20
