@@ -18,13 +18,13 @@
  * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
- * @(#) $Header: //depot/argus/argus/include/argus_ethertype.h#7 $ (LBL)
+ * @(#) $Header: //depot/argus/argus/include/argus_ethertype.h#11 $ (LBL)
  */
 
 /* 
- * $Id: //depot/argus/argus/include/argus_ethertype.h#7 $
- * $DateTime: 2011/01/26 17:11:49 $
- * $Change: 2087 $
+ * $Id: //depot/argus/argus/include/argus_ethertype.h#11 $
+ * $DateTime: 2015/04/02 15:28:09 $
+ * $Change: 2971 $
  */
 
 #ifndef  Argus_Ethertype_h
@@ -105,6 +105,9 @@ extern "C" {
 #ifndef ETHERTYPE_SCA
 #define ETHERTYPE_SCA		0x6007
 #endif
+#ifndef ETHERTYPE_TRANS_BRIDGE
+#define ETHERTYPE_TRANS_BRIDGE	0x6558
+#endif
 #ifndef ETHERTYPE_REVARP
 #define ETHERTYPE_REVARP	0x8035
 #endif
