@@ -1,6 +1,6 @@
 /*
  * Argus Software Common include files -  output structures
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2015 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus/argus/include/argus_out.h#37 $
- * $DateTime: 2012/01/03 19:15:19 $
- * $Change: 2274 $
+ * $Id: //depot/argus/argus/include/argus_out.h#52 $
+ * $DateTime: 2015/04/06 10:38:44 $
+ * $Change: 2973 $
  */
 
 
@@ -37,6 +37,8 @@
 #include <net/if.h>
 #endif
 
+#include <argus_util.h>
+
 #if defined(__OpenBSD__)
 #include <netinet/in.h>
 #endif
@@ -44,7 +46,6 @@
 #include <netinet/if_ether.h>
 #include <netinet/rtp.h>
 
-#include <argus_util.h>
 #include <argus_llc.h>
 #include <argus_isis.h>
 #include <argus_udt.h>
@@ -326,8 +327,6 @@ struct Argus80211Object {
    } o_un;
 };
 
-
-
 struct ArgusESPFlow {
    unsigned int ip_src, ip_dst;
    unsigned char ip_p, tp_p;
@@ -335,7 +334,6 @@ struct ArgusESPFlow {
    unsigned int spi;
 };
 
-
 struct ArgusESPv6Flow {
    unsigned int ip_src[4], ip_dst[4];
 #if defined(_LITTLE_ENDIAN)
@@ -591,7 +589,7 @@ struct ArgusEventTimeStruct {
 
 
 struct ArgusUniStats {
-   unsigned int pkts, bytes, appbytes;
+   long long pkts, bytes, appbytes;
 };
 
 struct ArgusStatObject {
@@ -751,6 +749,16 @@ struct ArgusLabelStruct {
    }l_un;
 };
 
+struct ArgusIdStruct {
+   int status;
+   char name[64];
+};
+
+struct ArgusInputStruct {
+   struct ArgusDSRHeader hdr;
+   struct ArgusIdStruct id;
+};
+
 
 struct ArgusEventStruct {
    struct ArgusDSRHeader       event;   /* immediate data */
@@ -778,6 +786,12 @@ struct ArgusMarStruct {
    unsigned int thisid, record_len;
 };
 
+struct ArgusMarSupStruct {
+   unsigned int status, argusid;
+   struct ArgusTime startime, now;
+   struct ArgusInputStruct input;
+};
+
 struct ArgusFarStruct {
    struct ArgusFlow flow;
 };
@@ -804,13 +818,15 @@ struct ArgusRecordHeader {
 struct ArgusRecord {
    struct ArgusRecordHeader hdr;
    union {
-      struct ArgusMarStruct mar;
-      struct ArgusFarStruct far;
-      struct ArgusEventStruct event;
+      struct ArgusMarStruct    mar;
+      struct ArgusMarSupStruct sup;
+      struct ArgusFarStruct    far;
+      struct ArgusEventStruct  event;
    } ar_un;
 };
 
 #define argus_mar       ar_un.mar
+#define argus_sup       ar_un.sup
 #define argus_far       ar_un.far
 #define argus_event     ar_un.event
 
@@ -880,16 +896,17 @@ struct ArgusRecordStruct {
    struct RaBinProcessStruct *bins;
    struct ArgusInput *input;
    struct ArgusDSRHeader *dsrs[ARGUSMAXDSRTYPE];
-   struct ArgusCorStruct *correlates;
-   float srate, drate, sload, dload, dur, mean;
-   float sploss, dploss;
-   long long offset;
 
    struct ArgusHashTableHdr *htblhdr;
    struct ArgusHashTableHdr *hinthdr;
    struct ArgusQueueStruct *nsq;
    struct ArgusRecordHeader hdr;
    struct ArgusCanonRecord canon;
+
+   struct ArgusCorStruct *correlates;
+   float srate, drate, sload, dload, dur, mean;
+   float pcr, sploss, dploss;
+   long long offset;
 };
 
 
